@@ -42,10 +42,11 @@ __license__   = "LGPL"
 import braillegenerator
 import debug
 import flat_review
+import formatting
 import keybindings
 import orca_state
 import settings
-import speechgenerator
+import speech_generator
 import structural_navigation
 import where_am_I
 import bookmarks
@@ -86,8 +87,10 @@ class Script:
         self.brailleBindings = self.getBrailleBindings()
         self.app_pronunciation_dict = self.getPronunciations()
 
+        self.formatting = self.getFormatting()
         self.brailleGenerator = self.getBrailleGenerator()
         self.speechGenerator = self.getSpeechGenerator()
+        self.generatorCache = {}
         self.whereAmI = self.getWhereAmI()
         self.bookmarks = self.getBookmarks()
         self.voices = settings.voices
@@ -178,6 +181,10 @@ class Script:
                 for command, handler in self.brailleBindings.iteritems()
                 if inputEventHandler == handler]
 
+    def getFormatting(self):
+        """Returns the formatting strings for this script."""
+        return formatting.Formatting(self)
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -186,7 +193,7 @@ class Script:
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
-        return speechgenerator.SpeechGenerator(self)
+        return speech_generator.SpeechGenerator(self)
 
     def getTutorialGenerator(self):
         """Returns the tutorial generator for this script.
@@ -334,6 +341,10 @@ class Script:
         if not processEvent:
             return
 
+        # Clear the generator cache for each event.
+        #
+        self.generatorCache = {}
+
         # This calls the first listener it finds whose key *begins with* or is
         # the same as the event.type.  The reason we do this is that the event
         # type in the listeners dictionary may not be as specific as the event
