@@ -100,7 +100,8 @@ class Script(default.Script):
             #print "COMPLETED", context.utterance, context.currentOffset
             obj = context.obj[len(context.obj)-1]
             obj.queryText().setCaretOffset(context.currentOffset)
-            orca.setLocusOfFocus(None, obj, False)
+            orca.setLocusOfFocus(
+                None, obj, notifyPresentationManager=False)
 
     def textLines(self, obj):
         """Creates a generator that can be used to iterate over each line
@@ -427,7 +428,7 @@ class Script(default.Script):
             line.addRegion(braille.Region(" " + label1))
             line.addRegion(braille.Region(" " + label2))
 
-            speech.speakUtterances(utterances)
+            speech.speak(utterances)
             braille.refresh()
 
     # This method tries to detect and handle the following cases:
@@ -569,8 +570,9 @@ class Script(default.Script):
                 # finding something.
                 #
                 speech.speak(_("Phrase found."))
-                utterances = self.speechGenerator.getSpeech(event.source, True)
-                speech.speakUtterances(utterances)
+                utterances = self.speechGenerator.generateSpeech(
+                    event.source, alreadyFocused=True)
+                speech.speak(utterances)
 
         # If Ctrl+G was used to repeat a find command, speak the line that
         # the caret moved to.
