@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2008 Sun Microsystems Inc.
+# Copyright 2005-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -31,10 +31,10 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: script.py 4674 2009-04-10 13:39:24Z wwalker $"
-__version__   = "$Revision: 4674 $"
-__date__      = "$Date: 2009-04-10 09:39:24 -0400 (Fri, 10 Apr 2009) $"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import gtk
@@ -51,10 +51,11 @@ import orca.settings as settings
 import orca.keybindings as keybindings
 
 from orca.orca_i18n import _ # for gettext support
+from orca.structural_navigation import StructuralNavigation
 
 from speech_generator import SpeechGenerator
 from braille_generator import BrailleGenerator
-from where_am_i import WhereAmI
+from formatting import Formatting
 import script_settings
 
 class Script(default.Script):
@@ -72,8 +73,12 @@ class Script(default.Script):
         #
         self.savedEnabledBrailledTextAttributes = None
         self.savedEnabledSpokenTextAttributes = None
-        self.speakCellCoordinatesCheckButton = None
+        self.speakSpreadsheetCoordinatesCheckButton = None
         self.savedreadTableCellRow = None
+        self.skipBlankCellsCheckButton = None
+        self.speakCellCoordinatesCheckButton = None
+        self.speakCellHeadersCheckButton = None
+        self.speakCellSpanCheckButton = None
 
         # Set the debug level for all the methods in this script.
         #
@@ -184,20 +189,25 @@ class Script(default.Script):
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
-
         return BrailleGenerator(self)
 
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
-
         return SpeechGenerator(self)
 
-    def getWhereAmI(self):
-        """Returns the "where am I" class for this script.
+    def getFormatting(self):
+        """Returns the formatting strings for this script."""
+        return Formatting(self)
+
+    def getEnabledStructuralNavigationTypes(self):
+        """Returns a list of the structural navigation object types
+        enabled in this script.
         """
 
-        return WhereAmI(self)
+        enabledTypes = [StructuralNavigation.TABLE_CELL]
+
+        return enabledTypes
 
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
@@ -207,6 +217,8 @@ class Script(default.Script):
         """
 
         default.Script.setupInputEventHandlers(self)
+        self.inputEventHandlers.update(\
+            self.structuralNavigation.inputEventHandlers)
 
         self.inputEventHandlers["speakInputLineHandler"] = \
             input_event.InputEventHandler(
@@ -298,6 +310,10 @@ class Script(default.Script):
                 self.inputEventHandlers["clearDynamicRowHeadersHandler"],
                 2))
 
+        bindings = self.structuralNavigation.keyBindings
+        for keyBinding in bindings.keyBindings:
+            keyBindings.add(keyBinding)
+
         return keyBindings
 
     def getAppPreferencesGUI(self):
@@ -316,12 +332,81 @@ class Script(default.Script):
         # column position within the spread sheet (i.e. A1, B1, C2 ...)
         #
         label = _("Speak spread sheet cell coordinates")
+        self.speakSpreadsheetCoordinatesCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.speakSpreadsheetCoordinatesCheckButton)
+        gtk.Box.pack_start(vbox, self.speakSpreadsheetCoordinatesCheckButton,
+                           False, False, 0)
+        gtk.ToggleButton.set_active(\
+            self.speakSpreadsheetCoordinatesCheckButton,
+            script_settings.speakSpreadsheetCoordinates)
+
+        # Table Navigation frame.
+        #
+        tableFrame = gtk.Frame()
+        gtk.Widget.show(tableFrame)
+        gtk.Box.pack_start(vbox, tableFrame, False, False, 5)
+
+        tableAlignment = gtk.Alignment(0.5, 0.5, 1, 1)
+        gtk.Widget.show(tableAlignment)
+        gtk.Container.add(tableFrame, tableAlignment)
+        gtk.Alignment.set_padding(tableAlignment, 0, 0, 12, 0)
+
+        tableVBox = gtk.VBox(False, 0)
+        gtk.Widget.show(tableVBox)
+        gtk.Container.add(tableAlignment, tableVBox)
+
+        # Translators: this is an option to tell Orca whether or not it
+        # should speak table cell coordinates in document content.
+        #
+        label = _("Speak _cell coordinates")
         self.speakCellCoordinatesCheckButton = gtk.CheckButton(label)
         gtk.Widget.show(self.speakCellCoordinatesCheckButton)
-        gtk.Box.pack_start(vbox, self.speakCellCoordinatesCheckButton,
+        gtk.Box.pack_start(tableVBox, self.speakCellCoordinatesCheckButton,
                            False, False, 0)
         gtk.ToggleButton.set_active(self.speakCellCoordinatesCheckButton,
-                                    script_settings.speakCellCoordinates)
+                                    settings.speakCellCoordinates)
+
+        # Translators: this is an option to tell Orca whether or not it
+        # should speak the span size of a table cell (e.g., how many
+        # rows and columns a particular table cell spans in a table).
+        #
+        label = _("Speak _multiple cell spans")
+        self.speakCellSpanCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.speakCellSpanCheckButton)
+        gtk.Box.pack_start(tableVBox, self.speakCellSpanCheckButton,
+                           False, False, 0)
+        gtk.ToggleButton.set_active(self.speakCellSpanCheckButton,
+                                    settings.speakCellSpan)
+
+        # Translators: this is an option for whether or not to speak
+        # the header of a table cell in document content.
+        #
+        label = _("Announce cell _header")
+        self.speakCellHeadersCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.speakCellHeadersCheckButton)
+        gtk.Box.pack_start(tableVBox, self.speakCellHeadersCheckButton,
+                           False, False, 0)
+        gtk.ToggleButton.set_active(self.speakCellHeadersCheckButton,
+                                    settings.speakCellHeaders)
+
+        # Translators: this is an option to allow users to skip over
+        # empty/blank cells when navigating tables in document content.
+        #
+        label = _("Skip _blank cells")
+        self.skipBlankCellsCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.skipBlankCellsCheckButton)
+        gtk.Box.pack_start(tableVBox, self.skipBlankCellsCheckButton,
+                           False, False, 0)
+        gtk.ToggleButton.set_active(self.skipBlankCellsCheckButton,
+                                    settings.skipBlankCells)
+
+        # Translators: this is the title of a panel containing options
+        # for specifying how to navigate tables in document content.
+        #
+        tableLabel = gtk.Label("<b>%s</b>" % _("Table Navigation"))
+        gtk.Widget.show(tableLabel)
+        gtk.Frame.set_label_widget(tableFrame, tableLabel)
+        gtk.Label.set_use_markup(tableLabel, True)
 
         return vbox
 
@@ -334,11 +419,30 @@ class Script(default.Script):
         """
 
         prefs.writelines("\n")
-        script_settings.speakCellCoordinates = \
-                 self.speakCellCoordinatesCheckButton.get_active()
         prefix = "orca.scripts.apps.soffice.script_settings"
-        prefs.writelines("%s.speakCellCoordinates = %s\n" % \
-                         (prefix, script_settings.speakCellCoordinates))
+
+        script_settings.speakSpreadsheetCoordinates = \
+                 self.speakSpreadsheetCoordinatesCheckButton.get_active()
+        prefs.writelines("%s.speakSpreadsheetCoordinates = %s\n" % \
+                         (prefix, script_settings.speakSpreadsheetCoordinates))
+
+        settings.speakCellCoordinates = \
+                 self.speakCellCoordinatesCheckButton.get_active()
+        prefs.writelines("orca.settings.speakCellCoordinates = %s\n" % \
+                         settings.speakCellCoordinates)
+
+        settings.speakCellSpan = self.speakCellSpanCheckButton.get_active()
+        prefs.writelines("orca.settings.speakCellSpan = %s\n" % \
+                         settings.speakCellSpan)
+
+        settings.speakCellHeaders = \
+                self.speakCellHeadersCheckButton.get_active()
+        prefs.writelines("orca.settings.speakCellHeaders = %s\n" % \
+                         settings.speakCellHeaders)
+
+        settings.skipBlankCells = self.skipBlankCellsCheckButton.get_active()
+        prefs.writelines("orca.settings.skipBlankCells = %s\n" % \
+                         settings.skipBlankCells)
 
     def getAppState(self):
         """Returns an object that can be passed to setAppState.  This
@@ -362,6 +466,27 @@ class Script(default.Script):
         except:
             debug.printException(debug.LEVEL_WARNING)
 
+    def isStructuralNavigationCommand(self, inputEvent=None):
+        """Checks to see if the inputEvent was a structural navigation
+        command. This is necessary to prevent double-presentation of
+        items. [[[TODO - JD: Perhaps this should be moved to default.py]]]
+
+        Arguments:
+        - inputEvent: The input event to examine. If none is provided,
+          orca_state.lastInputEvent will be used.
+
+        Returns True if inputEvent is a structural navigation command
+        enabled in this script.
+        """
+
+        inputEvent = inputEvent or orca_state.lastInputEvent
+        if isinstance(inputEvent, input_event.KeyboardEvent):
+            if self.structuralNavigation.keyBindings.\
+                    getInputHandler(inputEvent):
+                return True
+
+        return False
+
     def isReadOnlyTextArea(self, obj):
         """Returns True if obj is a text entry area that is read only."""
         state = obj.getState()
@@ -401,12 +526,13 @@ class Script(default.Script):
         isn't in a table.
         """
 
+        table = None
         obj = self.adjustForWriterTable(obj)
         if obj.getRole() == pyatspi.ROLE_TABLE_CELL and obj.parent:
             try:
                 table = obj.parent.queryTable()
             except NotImplementedError:
-                table = None
+                pass
 
         return table
 
@@ -546,13 +672,23 @@ class Script(default.Script):
         Returns True if this is a table cell, False otherwise.
         """
 
+        cell = obj
         if not startFromTable:
             obj = obj.parent
 
         try:
             table = obj.queryTable()
         except:
-            return False
+            # There really doesn't seem to be a good way to identify
+            # when the user is editing a cell because it has a role
+            # of paragraph and no table in the ancestry. This hack is
+            # a carry-over from the whereAmI code.
+            #
+            if cell.getRole() == pyatspi.ROLE_PARAGRAPH:
+                top = self.getTopLevel(cell)
+                return (top and top.name.endswith(" Calc"))
+            else:
+                return False
         else:
             return table.nRows == 65536
 
@@ -590,6 +726,31 @@ class Script(default.Script):
 
         return True
 
+    def findFrameAndDialog(self, obj):
+        """Returns the frame and (possibly) the dialog containing
+        the object. Overridden here for presentation of the title
+        bar information: If the locusOfFocus is a spreadsheet cell,
+        1) we are not in a dialog and 2) we need to present both the
+        frame name and the sheet name. So we might as well return the
+        sheet in place of the dialog so that the default code can do
+        its thing.
+        """
+
+        if not self.isSpreadSheetCell(obj):
+            return default.Script.findFrameAndDialog(self, obj)
+
+        results = [None, None]
+
+        parent = obj.parent
+        while parent and (parent.parent != parent):
+            if parent.getRole() == pyatspi.ROLE_FRAME:
+                results[0] = parent
+            if parent.getRole() == pyatspi.ROLE_TABLE:
+                results[1] = parent
+            parent = parent.parent
+
+        return results
+
     def printHierarchy(self, root, ooi, indent="",
                        onlyShowing=True, omitManaged=True):
         """Prints the accessible hierarchy of all children
@@ -718,8 +879,9 @@ class Script(default.Script):
             if newTable:
                 # We've entered a table.  Announce the dimensions.
                 #
-                line = _("table with %d rows and %d columns.") % \
-                        (newTable.nRows, newTable.nColumns)
+                line = _("table with %(rows)d rows and %(columns)d columns.") \
+                       % {"rows" : newTable.nRows,
+                          "columns" : newTable.nColumns}
                 speech.speak(line)
 
         if not newTable:
@@ -741,14 +903,7 @@ class Script(default.Script):
             return (cell != None)
 
         self.updateBraille(cell)
-        utterances = self.speechGenerator.getSpeech(cell, False)
-        if not len(utterances[0]) and self.speakBlankLine(newFocus):
-            # Translators: "blank" is a short word to mean the
-            # user has navigated to an empty line.
-            #
-            speech.speak(_("blank"), None, False)
-        else:
-            speech.speakUtterances(utterances)
+        speech.speak(self.speechGenerator.generateSpeech(cell))
 
         if not settings.readTableCellRow:
             self.speakCellName(cell.name)
@@ -1464,7 +1619,7 @@ class Script(default.Script):
         # box in Calc. If so, then replace the non-existent name with a
         # simple one before falling through and calling the default
         # locusOfFocusChanged method, which in turn will result in our
-        # _getSpeechForComboBox() method being called.
+        # _generateSpeechForComboBox() method being called.
         #
         rolesList = [pyatspi.ROLE_LIST,
                      pyatspi.ROLE_COMBO_BOX,
@@ -1496,9 +1651,9 @@ class Script(default.Script):
         if self.isSpreadSheetCell(event.source, True):
             if newLocusOfFocus:
                 self.updateBraille(newLocusOfFocus)
-                utterances = self.speechGenerator.getSpeech(newLocusOfFocus,
-                                                            False)
-                speech.speakUtterances(utterances)
+                utterances = \
+                    self.speechGenerator.generateSpeech(newLocusOfFocus)
+                speech.speak(utterances)
 
                 # Save the current row and column information in the table
                 # cell's table, so that we can use it the next time.
@@ -1534,9 +1689,9 @@ class Script(default.Script):
                     for tab in child:
                         eventState = tab.getState()
                         if eventState.contains(pyatspi.STATE_SELECTED):
-                            utterances = self.speechGenerator.getSpeech(tab,
-                                                                        False)
-                            speech.speakUtterances(utterances)
+                            utterances = \
+                                self.speechGenerator.generateSpeech(tab)
+                            speech.speak(utterances)
             # Fall-thru to process the event with the default handler.
 
         # If we are focused on a place holder element in the slide
@@ -1672,6 +1827,13 @@ class Script(default.Script):
         - event: the Event
         """
 
+        # If we've used structural navigation commands to get here, the
+        # presentation will be handled by the StructuralNavigation class.
+        # The subsequent event will result in redundant presentation.
+        #
+        if self.isStructuralNavigationCommand():
+            return
+
         # If this is a "focus:" event for the Calc Name combo box, catch
         # it here to reduce verbosity (see bug #364407).
         #
@@ -1685,7 +1847,7 @@ class Script(default.Script):
         if self.isDesiredFocusedItem(event.source, rolesList):
             debug.println(self.debugLevel, "StarOffice.onFocus - " \
                           + "Calc: Name combo box.")
-            orca.setLocusOfFocus(event, event.source, True)
+            orca.setLocusOfFocus(event, event.source)
             return
 
         # OOo Writer gets rather enthusiastic with focus: events for lists.
@@ -1715,7 +1877,7 @@ class Script(default.Script):
             # event.source lacks STATE_FOCUSED. This causes the default
             # script to ignore the event. See bug #523416. [[[TODO - JD:
             # If the OOo guys fix this on their end, this hack should be
-            # removed. The OOo issue can be found here: 
+            # removed. The OOo issue can be found here:
             # http://www.openoffice.org/issues/show_bug.cgi?id=93083]]]
             #
             rolesList = [pyatspi.ROLE_LIST,
@@ -1747,7 +1909,8 @@ class Script(default.Script):
         if handleEvent:
             if presentEvent:
                 speech.stop()
-            orca.setLocusOfFocus(event, event.any_data, presentEvent)
+            orca.setLocusOfFocus(
+                event, event.any_data, notifyPresentationManager=presentEvent)
 
             # We'll tuck away the activeDescendant information for future
             # reference since the AT-SPI gives us little help in finding
@@ -1827,8 +1990,7 @@ class Script(default.Script):
                 weToggledIt = wasCommand and keyString not in navKeys
 
             if weToggledIt:
-                speech.speakUtterances(self.speechGenerator.getSpeech( \
-                                       event.source, False))
+                speech.speak(self.speechGenerator.generateSpeech(event.source))
 
         # When a new paragraph receives focus, we get a caret-moved event and
         # two focus events (the first being object:state-changed:focused).
@@ -1848,7 +2010,8 @@ class Script(default.Script):
                          pyatspi.ROLE_ROOT_PANE,
                          pyatspi.ROLE_FRAME]
             if self.isDesiredFocusedItem(event.source, rolesList):
-                orca.setLocusOfFocus(event, event.source, False)
+                orca.setLocusOfFocus(
+                    event, event.source, notifyPresentationManager=False)
                 if event.source != self.currentParagraph:
                     self.updateBraille(event.source)
                 return
@@ -1860,7 +2023,8 @@ class Script(default.Script):
             #
             elif event.source.parent and \
                 event.source.parent.getRole() == pyatspi.ROLE_COMBO_BOX:
-                orca.setLocusOfFocus(None, event.source.parent, False)
+                orca.setLocusOfFocus(
+                    None, event.source.parent, notifyPresentationManager=False)
                 return
 
         # If we are in the sbase Table Wizard, try to reduce the numerous
@@ -1990,6 +2154,13 @@ class Script(default.Script):
         - event: the Event
         """
 
+        # If we've used structural navigation commands to get here, the
+        # presentation will be handled by the StructuralNavigation class.
+        # The subsequent event will result in redundant presentation.
+        #
+        if self.isStructuralNavigationCommand():
+            return
+
         # If we are losing focus and we in:
         # 1/ a paragraph in an ooimpress slide presentation
         # 2/ a paragraph in an oowriter text document
@@ -2235,7 +2406,7 @@ class Script(default.Script):
         # dialog, and loses focus requiring the user to know that it's
         # there and needs Alt+F6ing into.  But officially it's a normal
         # window.
- 
+
         # There doesn't seem to be (an efficient) top-down equivalent
         # of isDesiredFocusedItem(). But OOo documents have root panes;
         # this thing does not.
